/*
 *	klm_dispatch	This file dispatches RPC calls to the KLM server.
 *
 *	Cyopright (C) 1995, Olaf Kirch <okir@monad.swb.de>
 *
 *	03.06.95	okir
 *
 */

#include <stdio.h>
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "rpcdispatch.h"
#include "klm.h"

static struct dispatch_entry	dtable[] = {
	table_ent(klm_null,1,void,void),
	table_ent(klm_test,1,klm_testargs,klm_testrply),
	table_ent(klm_lock,1,klm_lockargs,klm_stat),
	table_ent(klm_cancel,1,klm_lockargs,klm_stat),
	table_ent(klm_unlock,1,klm_unlockargs,klm_stat),
};

typedef union {
	int			null;

	struct klm_testargs	test;
	struct klm_lockargs	lock;
	struct klm_lockargs	cancel;
	struct klm_unlockargs	unlock;
} klm_argument_union;

typedef union {
	int			voidres;
	klm_stat		stat;
	klm_testrply		testrply;
} klm_result_union;

static void
klm_dispatch(struct svc_req *rqstp, SVCXPRT *transp)
{
	klm_argument_union	argument;
	klm_result_union	result;

	if (rqstp->rq_proc != 0) {
		struct sockaddr_in *	caller;
		struct in_addr		addr;

		caller = svc_getcaller(rqstp->rq_xprt);
		addr = caller->sin_addr;
		if (ntohl(addr.s_addr) != INADDR_LOOPBACK) {
			log(L_ERROR, "klm call from %s rejected",
					inet_ntoa(addr));
			svcerr_weakauth(rqstp->rq_xprt);
			return;
		}
#ifndef DEBUG
		if (ntohs(caller->sin_port) >= IPPORT_RESERVED) {
			log(L_ERROR,
				"klm call from unprivileged port %d rejected",
				ntohs(caller->sin_port));
			svcerr_weakauth(rqstp->rq_xprt);
			return;
		}
#endif
	}

	rpc_dispatch(rqstp, transp,
			dtable, sizeof(dtable) / sizeof(dtable[0]),
			&argument, &result);
}

/*
 * Register the KLM server.
 */
void
klm_register(int prognum)
{
	SVCXPRT			*transp;

	if (prognum == 0)
		prognum = KLM_PROG;

	pmap_unset(prognum, 1);
	pmap_unset(prognum, 3);

	if ((transp = svcudp_create(RPC_ANYSOCK)) == NULL) {
		(void)fprintf(stderr, "cannot create udp service.\n");
		exit(1);
	}
	if (!svc_register(transp, prognum, 1, klm_dispatch, IPPROTO_UDP)) {
		fprintf(stderr, "Unable to register (KLM, 1, udp).\n");
		exit(1);
	}
	if (!svc_register(transp, prognum, 3, klm_dispatch, IPPROTO_UDP)) {
		fprintf(stderr, "Unable to register (KLM, 3, udp).\n");
		exit(1);
	}
}
