/*
 *	klmtest		simple tool for KLM testing.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include "klm_prot.h"
#include "unfsfh.h"

int makefh(char *path, netobj *fh);

int
main(int argc, char **argv)
{
	CLIENT		*client;
	klm_testargs	testargs;
	klm_lockargs	lockargs;
	klm_unlockargs	unlockargs;
	klm_testrply	*testres;
	klm_stat	*lockres;
	char		*filename = "foobar";
	char		*svchost = "localhost";
	unsigned long	offset = 0, length = 0;
	int		mypid = 1234;
	int		exclusive = 0;
	int		blocking = 0;
	int		unlock = 0;
	int		c;

	while ((c = getopt(argc, argv, "bf:h:l:o:p:ux")) != EOF) {
		switch(c) {
		case 'b':
			blocking = 1;
			break;
		case 'f':
			filename = optarg;
			break;
		case 'h':
			svchost = optarg;
			break;
		case 'l':
			length = atoi(optarg);
			break;
		case 'o':
			offset = atoi(optarg);
			break;
		case 'p':
			mypid = atoi(optarg);
			break;
		case 'u':
			unlock = 1;
			break;
		case 'x':
			exclusive = 1;
			break;
		default:
			fprintf(stderr, "klmtest: bad option %c\n", c);
			exit (2);
		}
	}

	client = clnt_create("localhost", KLM_PROG, KLM_VERS, "udp");
	if (client == NULL) {
		clnt_pcreateerror("localhost");
		exit(1);
	}


	testargs.exclusive = exclusive;
	testargs.alock.server_name = svchost;
	makefh(filename, &testargs.alock.fh);
	testargs.alock.pid = mypid;
	testargs.alock.l_offset = offset;
	testargs.alock.l_len = length;

	if ((testres = klm_test_1(&testargs, client)) == NULL) {
		clnt_perror(client, "klm_test call failed");
		exit (1);
	}
	printf ("klm_test reply:\n"
		"\tstatus:	%d\n",
			testres->stat
		);
	if (testres->stat == klm_denied) {
		klm_holder *holder = &(testres->klm_testrply_u.holder);
		printf ("\tconflicting lock:\n"
			"\tpid:		%d\n"
			"\toffset:	%d\n"
			"\tlength:	%d\n"
			"\texclusive:	%d\n",
			holder->svid,
			holder->l_offset,
			holder->l_len,
			holder->exclusive);
	}
	/*
	if (testres->stat != klm_granted && !unlock && !blocking)
		return 1;
	 */

	if (unlock) {
		unlockargs.alock.server_name = svchost;
		makefh(filename, &unlockargs.alock.fh);
		unlockargs.alock.pid = mypid;
		unlockargs.alock.l_offset = offset;
		unlockargs.alock.l_len = length;

		if ((lockres = klm_unlock_1(&unlockargs, client)) == NULL) {
			clnt_perror(client, "klm_unlock call failed");
			exit (1);
		}
		printf ("klm_unlock reply:\n"
			"\tstatus:	%d\n",
				lockres->stat
			);
	} else {
		lockargs.exclusive = exclusive;
		lockargs.alock.server_name = svchost;
		makefh(filename, &lockargs.alock.fh);
		lockargs.alock.pid = mypid;
		lockargs.alock.l_offset = offset;
		lockargs.alock.l_len = length;
		lockargs.block = blocking;

		if ((lockres = klm_lock_1(&lockargs, client)) == NULL) {
			clnt_perror(client, "klm_lock call failed");
			exit (1);
		}
		printf ("klm_lock reply:\n"
			"\tstatus:	%d\n",
				lockres->stat
			);
	}

	return 0;
}

int
makefh(char *path, netobj *fhp)
{
	static struct nfs_fh	fh;
	fhp->n_len = NFS_FHSIZE;
	if (!unfsfh_buildfh(path, &fh)) {
		fprintf(stderr, "couldn't make fh for %s\n", path);
		exit(1);
	}
	fhp->n_bytes = &fh;
	return 1;
}

