/*
 *	nlmtest		simple tool for NLM testing.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include "nlm_prot.h"


int
main(int argc, char **argv)
{
	CLIENT		*client;
	nlm_testargs	testargs;
	nlm_lockargs	lockargs;
	nlm_unlockargs	unlockargs;
	nlm_testres	*testres;
	nlm_res		*lockres;
	char		*filename = "foobar";
	char		*svchost = "localhost";
	unsigned long	offset = 0, length = 0;
	char		myhostname[257];
	int		hostnamelen, cookielen;
	char		ownerh[261];
	int		mypid = 1234;
	int		exclusive = 0;
	int		blocking = 0;
	int		unlock = 0;
	int		cookie = 4321;
	int		c;

	while ((c = getopt(argc, argv, "bf:h:l:o:p:ux")) != EOF) {
		switch(c) {
		case 'b':
			blocking = 1;
			break;
		case 'f':
			filename = optarg;
			break;
		case 'h':
			svchost = optarg;
			break;
		case 'l':
			length = atoi(optarg);
			break;
		case 'o':
			offset = atoi(optarg);
			break;
		case 'p':
			mypid = atoi(optarg);
			break;
		case 'u':
			unlock = 1;
			break;
		case 'x':
			exclusive = 1;
			break;
		default:
			fprintf(stderr, "nlmtest: bad option %c\n", c);
			exit (2);
		}
	}

	client = clnt_create(svchost, NLM_PROG, NLM_VERS, "udp");
	if (client == NULL) {
		clnt_pcreateerror("localhost");
		exit(1);
	}


	if (gethostname(myhostname, sizeof(myhostname)) < 0) {
		strcpy(myhostname, "unknown");
	}
	hostnamelen = strlen(myhostname);
	strcpy(ownerh, myhostname);
	memcpy(ownerh + hostnamelen + 1, &mypid, sizeof(mypid));
	cookielen = hostnamelen + 1 + sizeof(mypid);

	testargs.cookie.n_bytes = (void*)&cookie;
	testargs.cookie.n_len = sizeof(cookie);
	testargs.exclusive = exclusive;
	testargs.alock.caller_name = myhostname;
	testargs.alock.fh.n_len = strlen(filename);
	testargs.alock.fh.n_bytes = filename;
	testargs.alock.oh.n_len = cookielen;
	testargs.alock.oh.n_bytes = ownerh;
	testargs.alock.svid = mypid;
	testargs.alock.l_offset = offset;
	testargs.alock.l_len = length;

	if ((testres = nlm_test_1(&testargs, client)) == NULL) {
		clnt_perror(client, "nlm_test call failed:");
		exit (1);
	}
	printf ("nlm_test reply:\n"
		"\tcookie:	%d\n"
		"\tstatus:	%d\n",
			*(int*)(testres->cookie.n_bytes),
			testres->stat.stat
		);
	if (testres->stat.stat == nlm_denied) {
		nlm_holder *holder = &(testres->stat.nlm_testrply_u.holder);
		printf ("\tconflicting lock:\n"
			"\toh:		%s:%02x%2.2x%2.2x%2.2x\n"
			"\tpid:		%d\n"
			"\toffset:	%d\n"
			"\tlength:	%d\n"
			"\texclusive:	%d\n",
			holder->oh.n_bytes,
			holder->oh.n_bytes[holder->oh.n_len-4] & 0xFF,
			holder->oh.n_bytes[holder->oh.n_len-3] & 0xFF,
			holder->oh.n_bytes[holder->oh.n_len-2] & 0xFF,
			holder->oh.n_bytes[holder->oh.n_len-1] & 0xFF,
			holder->svid,
			holder->l_offset,
			holder->l_len,
			holder->exclusive);
	}
	if (testres->stat.stat != nlm_granted && !unlock && !blocking)
		return 1;

	if (unlock) {
		unlockargs.cookie.n_bytes = (void*)&cookie;
		unlockargs.cookie.n_len = sizeof(cookie);
		unlockargs.alock.caller_name = myhostname;
		unlockargs.alock.fh.n_len = strlen(filename);
		unlockargs.alock.fh.n_bytes = filename;
		unlockargs.alock.oh.n_len = cookielen;
		unlockargs.alock.oh.n_bytes = ownerh;
		unlockargs.alock.svid = mypid;
		unlockargs.alock.l_offset = offset;
		unlockargs.alock.l_len = length;

		if ((lockres = nlm_unlock_1(&unlockargs, client)) == NULL) {
			clnt_perror(client, "nlm_unlock call failed:");
			exit (1);
		}
		printf ("nlm_unlock reply:\n"
			"\tcookie:	%d\n"
			"\tstatus:	%d\n",
				*(int*)(lockres->cookie.n_bytes),
				lockres->stat.stat
			);
	} else {
		lockargs.cookie.n_bytes = (void*)&cookie;
		lockargs.cookie.n_len = sizeof(cookie);
		lockargs.exclusive = exclusive;
		lockargs.alock.caller_name = myhostname;
		lockargs.alock.fh.n_len = strlen(filename);
		lockargs.alock.fh.n_bytes = filename;
		lockargs.alock.oh.n_len = cookielen;
		lockargs.alock.oh.n_bytes = ownerh;
		lockargs.alock.svid = mypid;
		lockargs.alock.l_offset = offset;
		lockargs.alock.l_len = length;
		lockargs.block = blocking;
		lockargs.reclaim = 0;
		lockargs.state = 0;

		if ((lockres = nlm_lock_1(&lockargs, client)) == NULL) {
			clnt_perror(client, "nlm_lock call failed:");
			exit (1);
		}
		printf ("nlm_lock reply:\n"
			"\tcookie:	%d\n"
			"\tstatus:	%d\n",
				*(int*)(lockres->cookie.n_bytes),
				lockres->stat.stat
			);
	}

	return 0;
}

