/*
 * XDR routines
 *
 *	------	scd
 *
 */

#include "xdr.h"

int
xdr_share(XDR *xdrsp, struct locking *l)
{
   int  i;
   char g[32];
   char *p;

   /* Get length of cookie, MUST be 4 (until I work out how
    * to handle other cases)
    */
   if (!xdr_int(xdrsp, &l->cookie))
	return(0);
   if (l->cookie != 4)
	return(0);
   /* Get cookie as an unsigned int */
   if (!xdr_int(xdrsp, &l->cookie))
	return(0);

   p = l->name;
   if (!xdr_string(xdrsp, &p, 1024))
	return(0);
   p = l->fh;
   if (!xdr_bytes(xdrsp, &p, &i, 32))
	return(0);
   p = g;
   if (!xdr_bytes(xdrsp,&p,&i,32))
	return(0);
   if (!xdr_int(xdrsp, &l->mode))
	return(0);
   if (!xdr_int(xdrsp, &l->access))
	return(0);
   return(1);
}

/*
 *      DOS Share response.
 */
int
xdr_shareres(xdrsp,l)
        XDR *xdrsp;
        struct locking *l;
{
   int i;

   i = 4;

   /* Send cookie as unsigned int. Length must be 4 for now */
   if (!xdr_int(xdrsp, &i))
	return(0);
   if (!xdr_int(xdrsp, &l->cookie))
	return(0);

   if (!xdr_int(xdrsp,&l->stat))
	return(0);
   l->sequence++;
   if (!xdr_int(xdrsp,&l->sequence))
	return(0);
   return(1);
}

/*
 *      Lock request.
 */
int
xdr_lock(XDR *xdrsp, struct locking *l)
{
    int i;
    char *p;

    (void) memset(l, 0, sizeof(struct locking));

    /* Get cookie. Length must be 4 */
    if (!xdr_int(xdrsp, &l->opaque_len))
	return (0);
    if (!xdr_int(xdrsp, &l->cookie))
	return (0);

    /* blocking/exclusive */
    if (!xdr_bool(xdrsp, &l->block))
	return (0);
    if (!xdr_bool(xdrsp, &l->exclus))
	return (0);
    p = l->name;
    if (!xdr_string(xdrsp, &p, 1024))
	return (0);
    p = l->fh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);
    p = l->oh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);
    if (!xdr_int(xdrsp, &l->uppid))
	return (0);
    if (!xdr_u_long(xdrsp, &l->offset))
	return (0);
    if (!xdr_u_long(xdrsp, &l->length))
	return (0);
    if (!xdr_bool(xdrsp, &l->reclaim))
	return (0);
    if (!xdr_int(xdrsp, &l->state))
	return (0);
    return (1);
}


/*
 *      Unlock request.
*/
int
xdr_unlock(XDR *xdrsp, struct locking *l)
{
    int i;
    char *p;

    if (!xdr_int(xdrsp, &l->opaque_len))
	return (0);		/* Get length of cookie, MUST be 4 */
    if (!xdr_int(xdrsp, &l->cookie))
	return (0);		/* Get cookie as an unsigned int */

    p = l->name;
    if (!xdr_string(xdrsp, &p, 1024))
	return (0);
    p = l->fh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);
    p = l->oh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);		/* oh */
    if (!xdr_int(xdrsp, &i))
	return (0);		/* uppid */
    if (!xdr_u_long(xdrsp, &l->offset))
	return (0);
    if (!xdr_u_long(xdrsp, &l->length))
	return (0);
    return (1);
}

/*
 *      File Lock request.
 */
int
xdr_flock(XDR *xdrsp, struct locking *l)
{
    int i;
    char *p;

    (void) memset(l, 0, sizeof(struct locking));
    if (!xdr_int(xdrsp, &l->opaque_len))
	return (0);		/* Get length of cookie */
    if (!xdr_u_int(xdrsp, &l->cookie))
	return (0);		/* Get cookie as an unsigned int */

    if (!xdr_bool(xdrsp, &l->block))
	return (0);		/* block */
    if (!xdr_bool(xdrsp, &l->exclus))
	return (0);		/* exclusive */
    p = l->name;
    if (!xdr_string(xdrsp, &p, 1024))
	return (0);
    p = l->fh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);
    p = l->oh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);
    if (!xdr_int(xdrsp, &l->uppid))
	return (0);
    if (!xdr_u_long(xdrsp, &l->offset))
	return (0);
    if (!xdr_u_long(xdrsp, &l->length))
	return (0);
    if (!xdr_bool(xdrsp, &l->reclaim))
	return (0);
    if (!xdr_int(xdrsp, &l->state))
	return (0);
    return (1);
}

/*
 *      File Unlock request.
 */
int
xdr_funlock(XDR *xdrsp, struct locking *l)
{
    int i;
    char *p;

    (void) memset(l, 0, sizeof(struct locking));
    if (!xdr_int(xdrsp, &l->opaque_len))
	return (0);		/* Get length of cookie */
    if (!xdr_u_int(xdrsp, &l->cookie))
	return (0);		/* Get cookie as an unsigned int */

    p = l->name;
    if (!xdr_string(xdrsp, &p, 1024))
	return (0);
    p = l->fh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);
    p = l->oh;
    if (!xdr_bytes(xdrsp, &p, &i, 32))
	return (0);
    if (!xdr_int(xdrsp, &l->uppid))
	return (0);
    if (!xdr_u_long(xdrsp, &l->offset))
	return (0);
    if (!xdr_u_long(xdrsp, &l->length))
	return (0);
    return (1);
}

/*
 *      Send Lock response.
 */
int
xdr_res(XDR *xdrsp, struct locking *l)
{
    int i;

    i = 4;
    if (!xdr_int(xdrsp, &i))
	return (0);		/* Send length of cookie, MUST be 4 */
    if (!xdr_int(xdrsp, &l->cookie))
	return (0);		/* Send cookie as an unsigned int g */

    if (!xdr_int(xdrsp, &l->stat))
	return (0);
    return (1);
}

/*
 *      Remove all locks and shares for an owner.
 */
int
xdr_get_lock_name(XDR *xdrsp, char *l)
{
    return(xdr_string(xdrsp,&l,1024));
}

